if jit then
	bit = require "bit"
else
	bit = require "numberlua" -- For non-JIT, still super-slow. Thanks SiENcE!
end

require "gmb_opcodes"
require "gmb_cb_opcodes"
require "gmb_memory"
require "gameboy"
require "gmb_gpu"
require "gmb_interupts"
require "gmb_timers"
require "gmb_joypad"
require "gmb_sound"


function math.clamp(x, min, max)
  return x < min and min or (x > max and max or x)
end


function love.load()


	Options = {}
	Options.scale = 5
	Options.fullscreen = false
	Options.vsync = true

	Restart()

	ColourPalette = {}
	ColourPalette[3] = {0, 0, 0}		-- Black
	ColourPalette[2] = {112, 88, 72} 	-- Dark Grey
	ColourPalette[1] = {208, 168, 144} 	-- Light Grey
	ColourPalette[0] = {255, 255, 255}	-- White


	-- Load Resources
	love.graphics.setNewFont( "Gamegirl.ttf", 8 )
	gb_selection = love.graphics.newImage( "gb_selection.bmp" )


	love.graphics.setMode( 160 * Options.scale, 144 * Options.scale, Options.fullscreen, Options.vsync, 0 )

	canvas = love.graphics.newCanvas( 256, 256 )
	canvas:setFilter("nearest", "nearest")

	--oldframe = love.graphics.newCanvas( 256, 256 )
	--oldframe:setFilter("nearest", "nearest")


    love.graphics.setPoint( 1, "rough" )

    State = "main_menu"
    CurSelection = "Play ROM"


    UP = "up"
    DOWN = "down"
    LEFT = "left"
    RIGHT = "right"

    BUTTONA = "z"
    BUTTONB = "x"

    START = "return"
    SELECT = "backspace"

end

function loadROMList()

	CurPage = 1
    ROMList = {}
    TotalPages = 1
    CurSelection = 1

	ROMList = love.filesystem.enumerate( "roms" )

	for n = #ROMList, 1 do 
		if not love.filesystem.isFile( ROMList[n] ) then
			table.remove(ROMList, n)
		end
	end

	TotalPages = math.ceil( #ROMList/ 10 )

end



function love.update(dt) 
end

function love.keypressed(key)

	if State == "main_menu" then

		if CurSelection == "Play ROM" then

			if key == UP then CurSelection = "Options" 
			elseif key == DOWN then CurSelection = "Link Cable"
			elseif key == BUTTONA or key == START then State = "rom_selection", loadROMList()
			end

		elseif CurSelection == "Link Cable" then

			if key ==  UP then CurSelection = "Play ROM" 
			elseif key == DOWN then CurSelection = "Options" 
			end

		elseif CurSelection == "Options" then

			if key == UP then CurSelection = "Link Cable" 
			elseif key == DOWN then CurSelection = "Play ROM"
			elseif key == BUTTONA or key == START then State = "options" CurSelection = "Scale"
			end

		end

	elseif State == "rom_selection" then

		if key == UP then CurSelection = (CurSelection - 2)%10 + 1
		elseif key == DOWN then CurSelection = CurSelection%10 + 1
		elseif key == RIGHT then CurPage = CurPage%TotalPages + 1
		elseif key == LEFT then CurPage = (CurPage - 2)%TotalPages + 1
		elseif key == BUTTONB then State = "main_menu" CurSelection = "Play ROM"
		elseif key == BUTTONA or key == START then Restart() LoadRom("roms/" .. ROMList[ CurSelection + (CurPage - 1)*10 ]) State = "emulate"
		end

	elseif State == "options" then


		if key == BUTTONB then State = "main_menu" CurSelection = "Play ROM"

		elseif CurSelection == "Scale" then
			if key == UP then
			elseif key == DOWN then
			elseif key == LEFT then
				Options.scale = math.clamp ( Options.scale - 1, 1, 8 )
				love.graphics.setMode( 160 * Options.scale, 144 * Options.scale, Options.fullscreen, Options.vsync, 0 )
			elseif key == RIGHT then
				Options.scale = math.clamp ( Options.scale + 1, 1, 8 )
				love.graphics.setMode( 160 * Options.scale, 144 * Options.scale, Options.fullscreen, Options.vsync, 0 )

			end
		end



	end




end


function love.draw()

	--love.graphics.setCanvas( oldframe )
	    --love.graphics.setColor(255, 255, 255, 255) 
	    --love.graphics.draw( canvas, 0, 0)


	love.graphics.setCanvas( canvas )

	if State == "main_menu" then

		canvas:clear(255, 255, 255, 255)
		love.graphics.setColor(0, 0, 0, 255) 

		love.graphics.print("Play ROM", 20, 100)
		love.graphics.print("Link Cable", 20, 110)
		love.graphics.print("Options", 20, 120)

		if CurSelection == "Play ROM" then
			love.graphics.setColor(255, 255, 255, 255) 
			love.graphics.draw(gb_selection, 14, 100)



		elseif CurSelection == "Link Cable" then
			love.graphics.setColor(255, 255, 255, 255) 
			love.graphics.draw(gb_selection, 14, 110)


		elseif CurSelection == "Options" then
			love.graphics.setColor(255, 255, 255, 255) 
			love.graphics.draw(gb_selection, 14, 120)


		end

	elseif State == "options" then

		canvas:clear(255, 255, 255, 255)
		love.graphics.setColor(0, 0, 0, 255) 

		love.graphics.print( "Scale", 7, 5)

		love.graphics.setColor(ColourPalette[1][1], ColourPalette[1][2], ColourPalette[1][3], 255) 
		for n = 1, 8 do
			love.graphics.print( tostring(n), 50 + n*12, 5)
		end

		love.graphics.setColor(0, 0, 0, 255) 
		love.graphics.print( tostring(Options.scale), 50 + Options.scale*12, 5)



		love.graphics.setColor(0, 0, 0, 255) 
		love.graphics.print( "VSync", 7, 15)

		if Options.vsync then
			love.graphics.print( "Enabled", 90, 15)
		else
			love.graphics.setColor(ColourPalette[1][1], ColourPalette[1][2], ColourPalette[1][3], 255) 
			love.graphics.print( "Disabled", 90, 15)
		end




		love.graphics.setColor(0, 0, 0, 255) 
		love.graphics.print( "Fullscreen", 7, 25)


		if Options.fullscreen then
			love.graphics.print( "Enabled", 90, 25)
		else
			love.graphics.setColor(ColourPalette[1][1], ColourPalette[1][2], ColourPalette[1][3], 255) 
			love.graphics.print( "Disabled", 90, 25)
		end





	elseif State == "rom_selection" then

		canvas:clear(255, 255, 255, 255)
		love.graphics.setColor(0, 0, 0, 255) 

		for n = 1, 10 do
			love.graphics.print( ROMList[ n + (CurPage - 1)*10 ] or "", 7, n*10 - 5)
		end

		love.graphics.print( tostring(CurPage) .. " / " .. tostring(TotalPages), 60, 130 )

		love.graphics.setColor(255, 255, 255, 255) 
		love.graphics.draw(gb_selection, 1, CurSelection*10 - 5)


	elseif State == "emulate" then

		UpdateKeys()
		Think()

	end

    love.graphics.setCanvas()
	    love.graphics.setColor(255, 255, 255, 255) 
	    love.graphics.draw( canvas, 1, 1, 0, Options.scale, Options.scale)

	    --love.graphics.setColor(255, 255, 255, 128)
	    --love.graphics.draw( oldframe, 1, 1, 0, Options.scale, Options.scale)



    love.graphics.setCaption( love.timer.getFPS() .. " FPS   " .. (jit and "Jit" or "Interpreted") )

end
